
MailboxAlert.addToQueue = function (folder, message) {
    /* if this was just a move action, the message is probably not new */
    if (!message.isRead) {
        var cur_l = MailboxAlert.queue_length;
        var i = 0;
        for (i = 0; i < cur_l; i++) {
            if (MailboxAlert.queue_s[i] && MailboxAlert.queue[i] == folder) {
                /* set to new message */
                MailboxAlert.queue_message[i] = message;
                return;
            }
        }

        if (MailboxAlert.queue_length < MailboxAlert.max_queue_length) {
            dump("[mailboxalert] added folder to queue: ");
            dump(MailboxAlert.getFullFolderName(folder, true));
            dump("\r\n");
            MailboxAlert.queue[MailboxAlert.queue_length] = folder;
            MailboxAlert.queue_message[MailboxAlert.queue_length] = message;
            MailboxAlert.queue_s[MailboxAlert.queue_length] = true;
            MailboxAlert.queue_length++;
        } else {
            dump("[mailboxalert] max queue length (");
            dump(MailboxAlert.max_queue_length);
            dump(") reached (current queue ");
            dump(MailboxAlert.queue_length);
            dump(") , dropping message for: ");
            dump(MailboxAlert.getFullFolderName(folder, true));
            dump("\r\n");
        }
        if (!MailboxAlert.running) {
            MailboxAlert.running = true;
            var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
            try {
                var delay = prefs.getIntPref("extensions.mailboxalert.alert_delay");
                setTimeout('MailboxAlert.queueHandler()', delay * 1000);
            } catch (e) {
                setTimeout('MailboxAlert.queueHandler()', MailboxAlert.initial_wait_time);
            }
        }
    }
}

MailboxAlert.removeFromQueue = function (folder) {
    var cur_l = MailboxAlert.queue_length;
    var i = 0;
    for (i = 0; i < cur_l; i++) {
        if(MailboxAlert.queue_s[i] && MailboxAlert.queue[i] == folder) {
            dump("[mailboxalert] removing folder from queue: ");
            dump(MailboxAlert.getFullFolderName(folder, true));
            dump("\r\n");
            MailboxAlert.queue_s[i] = false;
            return;
        }
    }
    dump("[mailboxalert] unable to remove folder from queue, not present... ");
    dump(MailboxAlert.getFullFolderName(folder, true));
    dump("\r\n");
}

MailboxAlert.cleanQueue = function () {
    var cur_l = MailboxAlert.queue_length;
    var i = 0;
    var j;
    for (i = 0; i < cur_l; i++) {
        if (!MailboxAlert.queue_s[i]) {
            for (j = i + 1; j < cur_l; j++) {
                MailboxAlert.queue[j - 1] = MailboxAlert.queue[j];
                MailboxAlert.queue_s[j - 1] = MailboxAlert.queue_s[j];
                MailboxAlert.queue_message[j - 1] = MailboxAlert.queue_message[j];
            }
            cur_l--;
            i--;
        }
    }
    MailboxAlert.queue_length--;
    dump("[mailboxalert] done cleaning, queue length now ");
    dump(MailboxAlert.queue_length);
    dump("\r\n");
}

MailboxAlert.queueHandler = function () {
    MailboxAlert.running = true;
    if (MailboxAlert.queue_length > 0) {
        var i = 0;
        for (i = 0; i < MailboxAlert.queue_length; i++) {
            if (MailboxAlert.queue_s[i]) {
                var folder = MailboxAlert.queue[i];
                var last_unread = MailboxAlert.queue_message[i];

                if (!folder.gettingNewMessages) {
                    MailboxAlert.alert(folder, last_unread);

                    MailboxAlert.removeFromQueue(folder);

                    MailboxAlert.cleanQueue();
                } else {
                    dump("[mailboxalert] folder still getting: ");
                    dump(MailboxAlert.getFullFolderName(folder, true));
                    dump("\r\n");
                    // TODO: doesn't this break stuff?
                    folder.updateFolder(msgWindow);
                }
                
            }
        }
    }
    
    if (MailboxAlert.queue_length > 0) {
        setTimeout('MailboxAlert.queueHandler()', MailboxAlert.wait_time);
    } else {
        MailboxAlert.running = false;
    }
}

MailboxAlert.getHRMsgSize = function (messageBytes) {
    if (messageBytes) {
        if (messageBytes / 1073741824 >= 1) {
            return (Math.round(messageBytes / 1048576 * 100) / 100) + "G";
        } else if (messageBytes / 1048576 >= 1) {
            return (Math.round(messageBytes / 1048576 * 100) / 100) + "M";
        } else if (messageBytes / 1024 >= 1) {
            return (Math.round(messageBytes / 1024 * 100) / 100) + "K";
        } else {
            return messageBytes + "B";
        }
    } else {
        return "0B";
    }
}

MailboxAlert.escapeHTML = function (string) {
    if (string) {
        // the literal \n should be kept
        string = string.split("\n").join("%myentercode%");

        string = string.split("&").join("&#38;");
        string = string.split("\"").join("&#34;");
        string = string.split("'").join("&#39;");
        string = string.split("<").join("&#60;");
        string = string.split(">").join("&#62;");
        string = string.split("%myentercode%").join("\n");

        return string;
    } else {
        return "";
    }
}

MailboxAlert.getFullFolderName = function (folder, include_server) {
    var folderName = "";
    var i = 0;
    while(!folder.isServer && i < MailboxAlert.max_folder_depth) {
        folderName = folder.prettiestName + "/" + folderName;
        folder = folder.parent;
        i++;
    }
    if (include_server) {
        folderName = folder.prettiestName + "/" + folderName;
    }

    return folderName;
}

MailboxAlert.getServerName = function (folder) {
    var i = 0;
    var folderName;
    while(!folder.isServer && i < MailboxAlert.max_folder_depth) {
        folderName = folder.prettiestName + "/" + folderName;
        folder = folder.parent;
        i++;
    }
    return folder.prettiestName;
}

/* split-join method for replaceAll() */
MailboxAlert.replace = function (string, oldstr, newstr) {
    var replacement = "";
    if (newstr) {
        replacement = newstr;
    }
    if (string) {
        if (oldstr) {
            return String(string).split(oldstr).join(replacement);
        } else {
            return string;
        }
    } else {
        return "";
    }
}

/* split-join method for replaceAll() */
/* also escape spaces */
MailboxAlert.replaceEscape = function (string, oldstr, newstr) {
    var escaped_new = "";
    if (newstr) {
        escaped_new = newstr.split(" ").join("\\ ");
    }
    if (string) {
        if (oldstr) {
            return string.split(oldstr).join(escaped_new);
        } else {
            return string;
        }
    } else {
        return "";
    }
}

/* MailboxAlert.alert() checks whether the folder exists and can be used
 * MailboxAlert.alert2() checks whether the folder is not in use and has new mail
 * MailboxAlert.alert3() checks the settings and calls the actual alerts
 */
MailboxAlert.alert = function (folder, last_unread) {

    dump("[mailboxalert] alert called for ");
    dump(MailboxAlert.getFullFolderName(folder, true));
    dump("\r\n");

    //if (folder.flags & MSG_FOLDER_FLAG_VIRTUAL) {
    if (folder.flags & 0x0020) {
        var stringsBundle = document.getElementById("string-bundle");
        /* disable all alerts */
        var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
        var folder_name = MailboxAlert.getFullFolderName(folder, true);
        var folder_uri = folder.URI;
        alert(stringsBundle.getString('mailboxalert.error.virtualfolder') + " " + stringsBundle.getString('mailboxalert.error.disableallfor') + " " + folder_name);
        prefs.setBoolPref("extensions.mailboxalert.show_message." + folder_uri, false);
        prefs.setBoolPref("extensions.mailboxalert.play_sound." + folder_uri, false);
        prefs.setBoolPref("extensions.mailboxalert.execute_command." + folder_uri, false);

        return; // do nothing for saved searches
    }
    if (folder.flags) {
        MailboxAlert.alert2(folder, last_unread);
    } else {
        // skip new folders with no flag data
    }
}


/* MailboxAlert.alert() checks whether the folder exists and can be used
 * MailboxAlert.alert2() checks whether the folder is not in use and has new mail
 * MailboxAlert.alert3() checks the settings and calls the actual alerts
 */
MailboxAlert.alert2 = function (mailbox, last_unread) {
    var folder_name = MailboxAlert.getFullFolderName(mailbox, true);
    var folder_uri = mailbox.URI;
    if (folder_name) {
        dump("[mailboxalert] alert2 called for ");
        dump(folder_uri);
        dump("\r\n");

        try {
            /* message may have been filtered out already */
            if (mailbox == last_unread.folder) {
                if (last_unread.mime2DecodedAuthor) {
                    var folder = MailboxAlert.getFullFolderName(mailbox, false);
                    MailboxAlert.alert3(mailbox, null, folder, last_unread, false, 0);
                } else {
                    dump("mimedecode did not work, message probably moved");
                }
            } else {
            alert("message moved!");
            }
        } catch (e) {
            dump(e);
            dump("Stack trace:\n");
            dump(e.stack);
            dump("\n\n");
            var stringsBundle = document.getElementById("string-bundle");
            alert(stringsBundle.getString('mailboxalert.error')+"\r\n\r\n"+stringsBundle.getString('mailboxalert.error.exception')+":\r\n\r\n" + e + "\r\r\n\r\r\n" + stringsBundle.getString('mailboxalert.error.disableallfor') + " " + folder_name + "\n\n" + e.stack);
            var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
            prefs.setBoolPref("extensions.mailboxalert.show_message." + folder_uri, false);
            prefs.setBoolPref("extensions.mailboxalert.play_sound." + folder_uri, false);
            prefs.setBoolPref("extensions.mailboxalert.execute_command." + folder_uri, false);
        }
    } else {
        // ignore folders with no name
        dump("[mailboxalert] Folder has no name? skipping\r\n");
    }
}

/* MailboxAlert.alert() checks whether the folder exists and can be used
 * MailboxAlert.alert2() checks whether the folder is not in use and has new mail
 * MailboxAlert.alert3() checks the settings and calls the actual alerts
 */
MailboxAlert.alert3 = function (mailbox, orig_mailbox, orig_folder_name, last_unread, is_parent, orig_message_count) {

    var folder_name = MailboxAlert.getFullFolderName(mailbox, true);
    var folder_uri = mailbox.URI;
    if (folder_name) {
        dump("[mailboxalert] alert3 called for ");
        dump(folder_name);
        dump("\r\n");
        dump(folder_uri);
        dump("\r\n");

        var server = MailboxAlert.getServerName(mailbox);
        var new_message_count;
        if (is_parent) {
            new_message_count = orig_message_count;
        } else {
            new_message_count = mailbox.getNumUnread(false);
        }
        var all_message_count = mailbox.getNumUnread(true);

        // is there a more default default? TODO
        var charset = "ISO-8859-1";
        // not all folders have charset?
        // some messages have charset?
        try {
            charset = last_unread.Charset;
        } catch (e) {
            // ignore
        }

        if (charset == "ISO-8859-1") {
            try {
                charset = mailbox.charset;
            } catch (e) {
                // ignore
            }
        }

        // If no alert settings are set, we'll try the parent folder
        // so remember if we have alerted for this folder
        var alerted = false;

        var messageBytes = last_unread.messageSize;

        var messageKey = last_unread.messageKey;

        var subject = last_unread.mime2DecodedSubject;

        var sender = last_unread.mime2DecodedAuthor;
        var sender_name = sender;
        var sender_address = sender;
        if (sender.indexOf('<') > 0 && sender.indexOf('>') > 0) {
            sender_name = sender.substring(0, sender.indexOf('<'));
            sender_address = sender.substring(sender.indexOf('<') + 1, sender.indexOf('>'));
        }

        //var body_text = last_unread.nogwa
        var body_text = "";

        var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

        var alert_for_children = false;
        var no_alert_to_parent = false;

        try {
            alert_for_children = prefs.getBoolPref("extensions.mailboxalert.alert_for_children." + folder_uri);
        } catch (e) {
            // ok no command set for this mailbox
        }

        try {
            no_alert_to_parent = prefs.getBoolPref("extensions.mailboxalert.no_alert_to_parent." + folder_uri);
        } catch (e) {
            // ok no command set for this mailbox
        }
        dump("No alert to parent set to: " + no_alert_to_parent + "\n");

        dump("Is parent: " + is_parent + "\n");
        dump("Alert for children: " + alert_for_children + "\n");
        if (!is_parent || alert_for_children) {

            var show_message = false;
            var message = "";
            var subject_pref = "";

            var play_sound = false;
            var sound_default = true;
            var sound_wav = true;
            var sound_wav_file = "";

            var execute_command = false;
            var command = "";

            var escape = false;

            try {
                show_message = prefs.getBoolPref("extensions.mailboxalert.show_message." + folder_uri);
                message = prefs.getCharPref("extensions.mailboxalert.message."+folder_uri);
                subject_pref = prefs.getCharPref("extensions.mailboxalert.subject."+folder_uri);
            } catch (e) {
                // ok no command set for this mailbox
            }
            try {
                play_sound = prefs.getBoolPref("extensions.mailboxalert.play_sound." + folder_uri);
                sound_wav_file = prefs.getCharPref("extensions.mailboxalert.sound_wav_file."+folder_uri);
                sound_wav = prefs.getBoolPref("extensions.mailboxalert.sound_wav." + folder_uri);
            } catch (e) {
                // ok no command set for this mailbox
            }
            try {
                execute_command = prefs.getBoolPref("extensions.mailboxalert.execute_command." + folder_uri);
                command = prefs.getCharPref("extensions.mailboxalert.command."+folder_uri);
            } catch (e) {
                // ok no command set for this mailbox
            }
            try {
                escape = prefs.getBoolPref("extensions.mailboxalert.escape."+folder_uri);
            } catch (e) {
                // ok no command set for this mailbox
            }

			dump("Show message: " + show_message + "\n");
            if (show_message) {
                var show_icon = true;
                try {
                    var show_icon = prefs.getBoolPref("extensions.mailboxalert.show_message_icon."+folder_uri);
                } catch (e) {
                    // wasnt set
                }

                var icon_file = "chrome://mailboxalert/skin/mailboxalert.png";
                try {
                    icon_file = prefs.getCharPref("extensions.mailboxalert.icon_file."+folder_uri);
                } catch (e) {
                    // icon was not set
                }
                if (!orig_mailbox) {
					orig_mailbox = mailbox;
				}
                MailboxAlert.showMessage(last_unread.messageKey, orig_mailbox, server, folder_name, orig_folder_name, new_message_count, all_message_count, subject, sender, charset, show_icon, icon_file, subject_pref, message, messageBytes, last_unread);
                alerted = true;
            }

			dump("Play sound: " + play_sound + "\n");
            if (play_sound && !MailboxAlert.muted()) {
                if (sound_wav) {
                    dump("Play wav file: " + sound_wav_file + "\n");
                    MailboxAlert.playSound(sound_wav_file);
                } else {
                    dump("Play default system sound\n");
                    MailboxAlert.playSound("");
                }
                alerted = true;
            }

			dump("Execute command: " + execute_command + "\n");
			dump("Command: " + command + "\n");
            if (execute_command && command) {
                if (escape) {
                    server = MailboxAlert.escapeHTML(server);
                    orig_folder_name = MailboxAlert.escapeHTML(orig_folder_name);
                    subject = MailboxAlert.escapeHTML(subject);
                    sender = MailboxAlert.escapeHTML(sender);
                    sender_name = MailboxAlert.escapeHTML(sender_name);
                    sender_address = MailboxAlert.escapeHTML(sender_address);
                }

                MailboxAlert.executeCommand(server, folder_name, orig_folder_name, new_message_count, all_message_count, subject, sender, charset, command, messageBytes);
                alerted = true;
            }

        }

        dump("alerted: " + alerted);
        dump("\n");
        dump("isServer: " + mailbox.isServer);
        dump("\n");
        dump("is_parent: " + is_parent);
        dump("\n");
        dump("no_alert_to_parent: "+no_alert_to_parent);
        dump("\n");
        if (!alerted && !mailbox.isServer && !(!is_parent && no_alert_to_parent)) {
            dump("[Mailboxalert] No alerts were set for ");
            dump(folder_name);
            dump(", trying parent\r\n");
            if (!orig_mailbox) {
				orig_mailbox = mailbox;
			}
            MailboxAlert.alert3(mailbox.parent, orig_mailbox, orig_folder_name, last_unread, true, new_message_count);
        } else if (alerted) {
            dump("[mailboxalert] alerted for ");
            dump(folder_name);
            dump(" (original folder: ");
            dump(orig_folder_name);
            dump(")\r\n");
        } else if (!alerted) {
            if (no_alert_to_parent) {
                dump("[mailboxalert] no alert to parent set for ");
                dump(orig_folder_name);
                dump(")\r\n");
            } else {
                dump("[mailboxalert] no alert for top folder: ");
                dump(folder_name);
                dump(" (original folder: ");
                dump(orig_folder_name);
                dump(")\r\n");
            }
        }
    } else {
        dump("[mailboxalert] alert3: This folder has no name... skipping.\r\n");
    }
}

MailboxAlert.showMessage = function (message_key, mailbox, server, folder, orig_folder, new_message_count, all_message_count, subject, sender, charset, show_icon, icon_file, subject_pref, message, message_bytes, message_hdr) {
    
    var sender_name = sender;
    var sender_address = sender;
    var folder_url = mailbox.URI;

    if (sender.indexOf('<') > 0 && sender.indexOf('>') > 0) {
        sender_name = sender.substring(0, sender.indexOf('<'));
        var lastchar = sender_name.length - 1;
        while (sender_name[lastchar] == ' ') {
            lastchar = lastchar - 1;
        }
        if (sender_name[0] == '"' && sender_name[lastchar] == '"') {
            sender_name = sender_name.substring(1, lastchar);
        }
        sender_address = sender.substring(sender.indexOf('<') + 1, sender.indexOf('>'));
    }

    if (!message_bytes) {
        message_bytes = "0";
    }
    var messageSize = MailboxAlert.getHRMsgSize(message_bytes);

    
    subject_pref = MailboxAlert.replace(subject_pref, "%server", server);
    subject_pref = MailboxAlert.replace(subject_pref, "%originalfolder", orig_folder);
    subject_pref = MailboxAlert.replace(subject_pref, "%folder", folder);
    subject_pref = MailboxAlert.replace(subject_pref, "%countall", ""+all_message_count);
    subject_pref = MailboxAlert.replace(subject_pref, "%count", ""+new_message_count);
    subject_pref = MailboxAlert.replace(subject_pref, "%subject", subject);
    subject_pref = MailboxAlert.replace(subject_pref, "%senderaddress", sender_address);
    subject_pref = MailboxAlert.replace(subject_pref, "%sendername", sender_name);
    subject_pref = MailboxAlert.replace(subject_pref, "%sender", sender);
    subject_pref = MailboxAlert.replace(subject_pref, "%charset", charset);
    subject_pref = MailboxAlert.replace(subject_pref, "%messagebytes", message_bytes);
    subject_pref = MailboxAlert.replace(subject_pref, "%messagesize", messageSize);
    //subject_pref = MailboxAlert.replace(subject_pref, "%enter", "\n");

    var message_text = message;
    message_text = MailboxAlert.replace(message_text, "%server", server);
    message_text = MailboxAlert.replace(message_text, "%originalfolder", orig_folder);
    message_text = MailboxAlert.replace(message_text, "%folder", folder);
    message_text = MailboxAlert.replace(message_text, "%countall", ""+all_message_count);
    message_text = MailboxAlert.replace(message_text, "%count", ""+new_message_count);
    message_text = MailboxAlert.replace(message_text, "%subject", subject);
    message_text = MailboxAlert.replace(message_text, "%senderaddress", sender_address);
    message_text = MailboxAlert.replace(message_text, "%sendername", sender_name);
    message_text = MailboxAlert.replace(message_text, "%sender", sender);
    message_text = MailboxAlert.replace(message_text, "%charset", charset);
    message_text = MailboxAlert.replace(message_text, "%messagebytes", message_bytes);
    message_text = MailboxAlert.replace(message_text, "%messagesize", messageSize);
    message_text = MailboxAlert.replace(message_text, "%enter", "\n");

    try {
        window.openDialog('chrome://mailboxalert/content/newmailalert.xul', "new mail", "chrome,titlebar=no,popup=yes", subject_pref, message_text, show_icon, icon_file, mailbox, message_hdr);
    } catch (e) {
        alert(e);
    }
    


}

/* copied from mozilla thunderbird sourcecode */
MailboxAlert.playSound = function (soundURL) {
    var gSound = Components.classes["@mozilla.org/sound;1"].createInstance(Components.interfaces.nsISound);
    //gSound.init();
    if (soundURL) {
          if (soundURL.indexOf("file://") == -1) {
              soundURL = "file://" + soundURL;
          }
          try {
            var ioService = Components.classes["@mozilla.org/network/io-service;1"]
                     .getService(Components.interfaces.nsIIOService);
            var url = ioService.newURI(soundURL, null, null);
            dump("gSound.play("+url+")\n");
            gSound.play(url)
          } catch(e) {
              // some error, just 'beep' (which is system-dependent
              // these days)
              gSound.beep();
          }
    } else {
        gSound.beep();
    }
}

MailboxAlert.executeCommand = function (server, folder, orig_folder, new_message_count, all_message_count, subject, sender, charset, command, message_bytes) {
    var sender_name = sender;
    var sender_address = sender;
    if (sender.indexOf('<') > 0 && sender.indexOf('>') > 0) {
        sender_name = sender.substring(0, sender.indexOf('<'));
        var lastchar = sender_name.length - 1;
        while (sender_name[lastchar] == ' ') {
            lastchar = lastchar - 1;
        }
        if (sender_name[0] == '"' && sender_name[lastchar] == '"') {
            sender_name = sender_name.substring(1, lastchar);
        }
        sender_address = sender.substring(sender.indexOf('<') + 1, sender.indexOf('>'));
    }

    if (!message_bytes) {
        message_bytes = "0";
    }
    var messageSize = MailboxAlert.getHRMsgSize(message_bytes);


    command = MailboxAlert.replaceEscape(command, "%server", server);
    command = MailboxAlert.replaceEscape(command, "%originalfolder", orig_folder);
    command = MailboxAlert.replaceEscape(command, "%folder", folder);
    command = MailboxAlert.replaceEscape(command, "%countall", ""+all_message_count);
    command = MailboxAlert.replaceEscape(command, "%count", ""+new_message_count);
    command = MailboxAlert.replaceEscape(command, "%subject", subject);
    command = MailboxAlert.replaceEscape(command, "%senderaddress", sender_address);
    command = MailboxAlert.replaceEscape(command, "%sendername", sender_name);
    command = MailboxAlert.replaceEscape(command, "%sender", sender);
    command = MailboxAlert.replaceEscape(command, "%charset", charset);
    command = MailboxAlert.replace(command, "%messagebytes", message_bytes);
    command = MailboxAlert.replace(command, "%messagesize", messageSize);

    var args = new Array();
    var prev_i = 0;
    var i = 0;

    var env = Components.classes["@mozilla.org/process/environment;1"].createInstance(Components.interfaces.nsIEnvironment);
    var csconv = Components.classes["@mozilla.org/intl/saveascharset;1"].createInstance(Components.interfaces.nsISaveAsCharset);
    var tocharset = env.get("LANG").split(".")[1];

    dump("Command to execute: ");
    dump(command);
    dump("\n");
    var i = 0;
    for (i; i < command.length; i++) {
        if (command.substr(i, 1) == " ") {
            if (i > 0 && command.substr(i-1, 1) != "\\") {
                var cur_arg = command.substring(prev_i, i);
                // remove escapes again
                cur_arg = cur_arg.split("\\ ").join(" ");

                /* convert every argument (i.e. everything but the
                 * first) to native charset */
                if (prev_i > 0) {
                    if (charset && tocharset) {
                        csconv.Init(tocharset, 0, 0);
                        cur_arg = csconv.Convert(cur_arg);
                    }
                }
                args.push(cur_arg);
                prev_i = i + 1;
            } else {
                //alert("space at pos "+i+" is escaped: "+command.substr(i-1, 1));
            }
        }
    }
    /* also convert this one */
    if (charset && tocharset) {
        csconv.Init(tocharset, 0, 0);
        args.push(csconv.Convert(command.substr(prev_i, i).split("\\ ").join(" ")));
    } else {
        args.push(command.substr(prev_i, i).split("\\ ").join(" "));
    }
    var executable_name = args.shift();
    dump("Executable: ");
    dump(executable_name);
    dump("\n");
    try {
        var exec = Components.classes["@mozilla.org/file/local;1"].
        createInstance(Components.interfaces.nsILocalFile);
        var pr = Components.classes["@mozilla.org/process/util;1"].
        createInstance(Components.interfaces.nsIProcess);

        exec.initWithPath(executable_name);
        // isExecutable is horribly broken in OSX, see
        // https://bugzilla.mozilla.org/show_bug.cgi?id=322865
        // So use a fugly os detection here...
        if (!exec.exists() || !(/Mac/.test(navigator.platform) || exec.isExecutable()) || !exec.isFile()) {
            var stringsBundle = document.getElementById("string-bundle");
            alert(stringsBundle.getString('mailboxalert.error')+"\n" + exec.leafName + " " + stringsBundle.getString('mailboxalert.error.notfound') + "\n\nFull path: "+executable_name+"\n\n" + stringsBundle.getString('mailboxalert.error.disableexecutefor') + " " + folder);
            dump("Failed command:  " +executable_name + "\r\n");
            dump("Arguments: " + args + "\r\n");
                    var caller = window.arguments[0];
            if (caller) {
                var executecommandcheckbox = document.getElementById('mailboxalert_execute_command');
                executecommandcheckbox.checked = false;
                setUIExecuteCommandPrefs(false);
            } else {
                var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
                prefs.setBoolPref("extensions.mailboxalert.execute_command." + folder, false);
            }
        } else {
            dump("Command:  " +executable_name + "\r\n");
            dump("Arguments: " + args + "\r\n");
            var res1 = pr.init(exec);
            var result = pr.run(false, args, args.length);
        }
    } catch (e) {
        if (e.name == "NS_ERROR_FILE_UNRECOGNIZED_PATH") {
            var stringsBundle = document.getElementById("string-bundle");
            alert(stringsBundle.getString('mailboxalert.error') + "\r\n\r\n" +
                  stringsBundle.getString('mailboxalert.error.badcommandpath1') + 
                  " " + folder + " " +
                  stringsBundle.getString('mailboxalert.error.badcommandpath2')  +
                  "\r\n\r\n" +
                  stringsBundle.getString('mailboxalert.error.disableexecutefor') + " " + folder);
                    var caller = window.arguments[0];
            if (caller) {
                var executecommandcheckbox = document.getElementById('mailboxalert_execute_command');
                executecommandcheckbox.checked = false;
                setUIExecuteCommandPrefs(false);
            } else {
                var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
                prefs.setBoolPref("extensions.mailboxalert.execute_command." + folder, false);
            }
        } else {
            throw e;
        }
    }
}
